<?php
/**
 * A factory class for cities
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class CityDaemonFactory extends ObjectFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton (registry-based) CityHelper instance
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	$cityInstance		An instance of a CityHelper class
	 * @throws	FactoryRequiredParameterException	If not all parameters are set and no instance 'city' is set.
	 */
	public static final function createCityDaemonInstance (Requestable $requestInstance = NULL, Responseable $responseInstance = NULL) {
		// Get new factory instance
		$factoryInstance = new CityDaemonFactory();

		// If there is no handler?
		if (Registry::getRegistry()->instanceExists('city')) {
			// Get handler from registry
			$cityInstance = Registry::getRegistry()->getInstance('city');
		} elseif (($requestInstance instanceof Requestable) && ($responseInstance instanceof Responseable)) {
			// The default city-mode is from our configuration
			$cityMode = $factoryInstance->getConfigInstance()->getConfigEntry('city_default_mode');

			// Is the city 'mode' parameter set?
			if ($requestInstance->isRequestElementSet('mode')) {
				// Then use this which overrides the config entry temporarily
				$cityMode = $requestInstance->getRequestElement('mode');
			} else {
				// Set it for easier re-usage
				$requestInstance->setRequestElement('mode', $cityMode);
			}

			// Now convert the city-mode in a class name
			$className = 'Simulation' . $factoryInstance->convertToClassName($cityMode) . 'CityDaemon';

			// Get the city instance
			$cityInstance = ObjectFactory::createObjectByName($className, array($requestInstance));

			// Get a registry
			$applicationInstance = Registry::getRegistry()->getInstance('app');

			// Set the app instance
			$cityInstance->setApplicationInstance($applicationInstance);

			// Add city-specific filters
			$cityInstance->addExtraFilters($applicationInstance->getControllerInstance(), $responseInstance);
		} else {
			// Throw an exception here
			throw new FactoryRequiredParameterException($factoryInstance, self::EXCEPTION_FACTORY_REQUIRE_PARAMETER);
		}

		// Return the instance
		return $cityInstance;
	}
}

// [EOF]
?>
